/*
 * Copyright (C) 2018 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: Denis Osvald <denis.osvald@sartura.hr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#include <stdio.h>
#include <string.h>
#include <stdbool.h>

#include "utils.h"

int ubus_object_path_compare(const void *k1, const void *k2, void *ptr)
{
	const char *pattern1 = (char *)k1;
	const char *pattern2 = (char *)k2;
	size_t len1 = strlen(pattern1);
	size_t len2 = strlen(pattern2);
	bool p1_wildcard, p2_wildcard;

	if (len1 == 0 || len2 == 0)
		return len1 - len2;
	p1_wildcard = (pattern1[len1-1] == '*');
	p2_wildcard = (pattern2[len2-1] == '*');

	/* none has wildcard */
	if (!p1_wildcard && !p2_wildcard)
		return strcmp(pattern1, pattern2);

	/* only one pattern has wildcard */
	if (p1_wildcard != p2_wildcard) {
		return strncmp(pattern1, pattern2,
				(p1_wildcard ? len1 : len2) - 1);
	}

	/* both have wildcard */
	return strncmp(pattern1, pattern2, (len1 < len2 ? len1 : len2) - 1);
}

